<?php

/**
 * Classe de requêtes HTTP
 *
 * @author Yvan Taviaud
 * @see http://www.labsmedia.com/index.html
 * @since 09/04/2005
 */
class Http
{

	/** Mode débug */
	private $debugVars = array(true, '');
	/* Erreur éventuelle */
	public $error;
	/* Erreur du socket */
	public $socketError;
	public $socketErrNo;
	/* Code de retour */
	public $status;
	/* Contenu renvoyé */
	public $contents;
	/* Headers */
	public $headers;
	/* Limite de redirections */
	public $redirects = 3;
	/* Timeout de réception d'un paquet */
	public $timeout = 5;
	/* Temps en microseconde à attendre avant une nouvelle requete sur le hosts */
	public $usleep = 0;
	/* Vérification de l'espace mémoire disponible (test sur le Content-Length uniquement) */
	public $memory = false;
	/* Taille du tampon par host pour les microsleep */
	const hostBufferSize = 100;

	/** @var array $connections Connections pour Keep-Alive */
	private $connections = array();
	/** @var object $_instance	Instance d'objet (singleton) */
	private static $_instance;

	/**
	 * Constructeur privé
	 */
	private function __construct()
	{
		
	}

	/**
	 * Retourne l'instance de l'http
	 *
	 * @return Http
	 */
	public static function getInstance()
	{
		if (!isset(self::$_instance))
		{
			self::$_instance = new Http();
		}
		return self::$_instance;
	}

	/**
	 * Retourne l'IP d'un hôte. Cela va bien plus vite que de laisser la tâche à fsockopen
	 *
	 * @param string $host
	 * @return mixed
	 */
	private function hostToIp($host)
	{
		static $ips = array();
		if (isset($ips[$host]))
		{
			return $ips[$host];
		}
		if (ip2long($host) !== false)
		{
			/* C'est une IP, on fait comme si c'était un host pour ne pas se poser de questions :-) */
			$ip = $host;
		}
		else
		{
			$ip = gethostbyname($host);
			if ($ip === $host || $ip === false)
			{
				return false;
			}
		}
		$ips[$host] = $ip;
		return $ip;
	}

	/**
	 * Récupère une connexion si celle-ci a échoué
	 *
	 * @param string $host
	 * @param integer $port
	 * @param boolean $force
	 * @return resource
	 */
	private function getConnection($host, $port, $force = false)
	{
		/* Tampon des dernieres connexions par host */
		static $hostBuffer = array();
		/* Nombre de hosts dans le tampon */
		static $hostCount = 0;
		/* Si on a activé les microsleeps */
		if ($this->usleep > 0)
		{
			/* Dernière connexion pour ce host */
			$lastConnection = &$hostBuffer[$host];
			/* Si on a deja eu une connexion pour ce host */
			if (isset($lastConnection) === false)
			{
				/* Si on a atteint la taille limite du tampon */
				if ($hostCount > self::hostBufferSize)
				{
					/* On supprime le premier, FIFO powered */
					list($deleteHost, ) = each($hostBuffer);
					unset($hostBuffer[$deleteHost]);
				}
				/* Si on ne supprime pas c'est qu'on ajoute */
				else
				{
					$hostCount++;
				}
				$lastConnection = microtime(true);
			}
			/* Si le temps entre les deux requêtes est trop court on fait une pause */
			elseif (microtime(true) < $lastConnection + $this->usleep)
			{
				usleep($this->usleep);
			}
		}
		$now = time();
		$conn = &$this->connections[$host.':'.$port];
		if (isset($conn) && is_resource($conn['c']) && $force === false && !feof($conn['c'])
				&& ($conn['t'] === 0 || $conn['t'] > $now)) // Keep-Alive timeout
		{
			return $conn['c'];
		}
		$ip = $this->hostToIp($host);
		if ($ip === false)
		{
			return false;
		}
		/* Paramètres de la connexion */
		$conn = array('c' => false, 's' => $now, 't' => 0);
		/* Choix du mode (SSL ou pas) */
		if ($port === 443)
		{
			$conn['c'] = @fsockopen('ssl://'.$ip, 443, $this->socketErrNo, $this->socketError, $this->timeout);
		}
		else
		{
			$conn['c'] = @fsockopen($ip, $port, $this->socketErrNo, $this->socketError, $this->timeout);
		}
		if ($this->debugVars[0] === false)
		{
			$this->debugVars[1] .= microtime(true).'     FSOCKOPEN: '.$ip.':'.$port.', timeout='.$this->timeout."\n";
		}
		return $conn['c'];
	}

	private function check($connection)
	{
		if (!isset($this->connections[$connection]))
		{
			return true;
		}
		if (isset($this->headers['Connection']) && $this->headers['Connection'] === 'close')
		{
			unset($this->connections[$connection]);
			return true;
		}
		if (isset($this->headers['Keep-Alive']) && $this->connections[$connection]['t'] === 0)
		{
			if (($pos = strpos($this->headers['Keep-Alive'], 'timeout=')) !== false)
			{
				$this->connections[$connection]['t'] = $this->connections[$connection]['s'] + (int) substr($this->headers['Keep-Alive'], $pos + 8);
			}
		}
	}

	/**
	 * Requête GET
	 *
	 * @param string $url URL à charger
	 * @param string $queryString Chaîne contenant les paramètres à passer à la requête
	 * @return boolean Résultat de la requête (true = 200 OK)
	 */
	function get($url, $queryString = '', $headers = array())
	{
		return $this->request($url, 'GET', $queryString, $headers);
	}

	/**
	 * Requête POST
	 *
	 * @param string $url URL à charger
	 * @param string $queryString Chaîne contenant les paramètres à passer à la requête
	 * @return boolean Résultat de la requête (true = 200 OK)
	 */
	function post($url, $queryString = '', $headers = array())
	{
		return $this->request($url, 'POST', $queryString, $headers);
	}

	/**
	 * Requête HEAD
	 *
	 * @param string $url URL à charger
	 * @param string $queryString Chaîne contenant les paramètres à passer à la requête
	 * @return boolean Résultat de la requête (true = 200 OK)
	 */
	function head($url, $queryString = '', $headers = array())
	{
		return $this->request($url, 'HEAD', $queryString, $headers);
	}

	/**
	 * Requête PURGE
	 *
	 * @param string $url URL à purger
	 * @param string $queryString Chaîne contenant les paramètres à passer à la requête
	 * @return boolean Résultat de la requête (true = 200 OK)
	 */
	function purge($url, $queryString = '', $headers = array())
	{
		return $this->request($url, 'PURGE', $queryString, $headers);
	}

	/**
	 * Requête Principale
	 *
	 * @param string $url URL à charger
	 * @param string $requestType Type de la requête (GET/POST)
	 * @param string $queryString Chaîne contenant les paramètres à passer à la requête
	 * @param array $headers Tableau des headers à envoyer
	 * @return boolean Résultat de la requête (true = 200 OK)
	 */
	private function request($url, $requestType, $queryString = '', $headers = array())
	{
		$this->status = 0;
		$this->error = '';
		$this->contents = '';
		/* On conserve les cookies originaux, dans le cas d'une redirection 302 qui les perdrait au passage. Et on ajoute la dernière url téléchargée */
		$this->headers = array('Set-Cookie' => array(), 'Cookies' => isset($headers['Cookie']) ? $headers['Cookie'] : '', 'X-Last-Location' => $url);

		/* Explosion de l'URL */
		$parsedUrl = @parse_url($url);
		if ($parsedUrl === false || !isset($parsedUrl['host']))
		{
			return false;
		}

		if (!isset($parsedUrl['path']) || $parsedUrl['path'] === '')
		{
			$parsedUrl['path'] = '/';
		}
		if (!isset($parsedUrl['port']))
		{
			if (isset($parsedUrl['scheme']) && $parsedUrl['scheme'] === 'https')
			{
				$parsedUrl['port'] = 443;
			}
			else
			{
				$parsedUrl['port'] = 80;
			}
		}
		/* Zut, on a passé une partie des infos dans l'url ! */
		if (isset($parsedUrl['query']) && $parsedUrl['query'] !== '')
		{
			if ($requestType !== 'POST')
			{
				if ($queryString !== '')
				{
					$queryString .= '&';
				}
				$queryString .= $parsedUrl['query'];
			}
			else
			{
				$parsedUrl['path'] = $parsedUrl['path'].'?'.$parsedUrl['query'];
			}
		}
		/* Préparation des headers : on prétend être un vrai navigateur */
		$defaultHeaders = array('User-Agent' => 'Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.14) Gecko/20110218 Firefox/3.6.14', 'Accept' => 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8', 'Accept-Language' => 'en, en_US, fr, fr_FR', 'Accept-Charset' => 'ISO-8859-1,utf-8;q=0.7,*;q=0.7', 'X-Moz' => 'prefetch', 'Keep-Alive' => '300');
		if (function_exists('gzinflate'))
		{
			$defaultHeaders['Accept-Encoding'] = 'gzip,deflate';
		}
		if ($requestType === 'POST')
		{
			$defaultHeaders['Content-Type'] = 'application/x-www-form-urlencoded';
			$defaultHeaders['Content-Length'] = strlen($queryString);
		}
		$defaultHeaders['Connection'] = 'keep-alive';
		$headers = array_merge($defaultHeaders, $headers);

		/* Utilisation d'une IP définie */
		if (isset($headers['useIp']))
		{
			if ($headers['useIp'] !== '')
			{
				$host = $headers['useIp'];
			}
			else
			{
				$host = $parsedUrl['host'];
			}
			unset($headers['useIp']);
		}
		else
		{
			$host = $parsedUrl['host'];
		}
		/* Arrêt juste après les entêtes */
		if (isset($headers['headersOnly']) || $requestType === 'HEAD')
		{
			$headersOnly = true;
			unset($headers['headersOnly']);
		}
		else
		{
			$headersOnly = false;
		}
		$fputs = '';
		$fputs .= $requestType.' '.$parsedUrl['path'];
		if ($requestType !== 'POST' && $queryString !== '')
		{
			$fputs .= '?'.$queryString;
		}
		$fputs .= " HTTP/1.1\r\nHost: ".$parsedUrl['host']."\r\n";
		foreach ($headers as $header => $value)
		{
			if (!empty($value))
			{
				$fputs .= $header.': '.$value."\r\n";
			}
		}
		$fputs .= "\r\n";
		if ($requestType === 'POST')
		{
			$fputs .= $queryString;
		}
		if ($this->debugVars[0] === false)
		{
			$this->debugVars[1] .= microtime(true).'   CALL: '.$host.':'.$parsedUrl['port']." : \n".microtime(true)." HEADER: ";
			$this->debugVars[1] .= str_replace("\r\n", "\n".microtime(true)." HEADER: ", $fputs);
			$this->debugVars[1] .= "\n-------------\n";
		}
		$conn = $this->getConnection($host, $parsedUrl['port']);
		$res = @fputs($conn, $fputs);
		if ($res === 0 || $res === false)
		{
			/* La connexion était en Keep-Alive certainement, on retente une nouvelle connexion */
			$conn = $this->getConnection($host, $parsedUrl['port'], true);
			$res = @fputs($conn, $fputs);
			if ($res === 0 || $res === false)
			{
				$this->error = 'fputs';
				return false;
			}
		}
		$buffer = '';
		if ($this->debugVars[0] === false)
		{
			$timer = time();
			$this->debugVars[1] .= microtime(true).'  STREAM_STATE: '.gettype($conn)."\n";
		}
		/* Time out de lecture */
		stream_set_timeout($conn, $this->timeout);
		if ($this->debugVars[0] === false)
		{
			$this->debugVars[1] .= microtime(true).'  STREAM_STATE: '.gettype($conn).' ('.(time() - $timer).'/'.$this->timeout." sec)\n";
		}
		/* Lecture de la réponse HTTP */
		while (feof($conn) === false)
		{
			$buffer = fgets($conn);
			if ($this->debugVars[0] === false)
			{
				$this->debugVars[1] .= microtime(true).'   ANSWER_HTTP: '.($buffer === false ? 'FALSE: '.$this->socketError.' - '.$this->socketErrNo : trim($buffer))."\n";
			}
			if ($buffer === false)
			{
				$this->error = 'stream_error/timeout - '.$this->socketError.' - '.$this->socketErrNo;
				return false;
			}
			$pos = strpos($buffer, 'HTTP/1.');
			if ($pos !== false)
			{
				$this->status = (int) substr($buffer, $pos + 9, 3);
				if ($this->status !== 0 && $this->status !== 100)
				{
					break;
				}
			}
		}
		if ($this->status === 0)
		{
			$this->error = 'no_status_code';
			return false;
		}
		/* Lecture des headers */
		while (feof($conn) === false)
		{
			$buffer = fgets($conn);
			if ($buffer === false)
			{
				$this->error = 'buffer';
				return false;
			}
			if ($this->debugVars[0] === false)
			{
				$this->debugVars[1] .= microtime(true).' ANSWER_HEADER: '.trim($buffer)."\n";
			}
			$buffer = trim($buffer);
			$colonPos = strpos($buffer, ':');
			if ($colonPos !== false)
			{
				$headerName = substr($buffer, 0, $colonPos);
				if ($headerName === 'Set-Cookie')
				{
					if (preg_match('~^\s*(.*?)=(.*?);~i', rawurldecode(substr($buffer, $colonPos + 2)).';', $cookie) === 1)
					{
						/* Si on trouve 2 fois le même cookie, on doit mettre à jour la chaîne des cookies pour supprimer le premier */
						if (isset($this->headers[$headerName][$cookie[1]]))
						{
							$this->headers['Cookies'] = preg_replace('/'.$cookie[1].'=.*?; /', '', $this->headers['Cookies']);
						}
						$this->headers[$headerName][$cookie[1]] = $cookie[2];
						$this->headers['Cookies'] .= $cookie[1].'='.$cookie[2].'; ';
					}
				}
				else
				{
					$this->headers[$headerName] = substr($buffer, $colonPos + 2);
				}
			}
			if ($buffer === '')
				break;
		}
		if ($this->debugVars[0] === false)
		{
			$this->debugVars[1] .= "-------------\n";
		}
		/* Vérification de la connexion, maintenant qu'on a toutes les infos du Keep-Alive */
		$this->check($parsedUrl['host'].':'.$parsedUrl['port']);

		/* Saletés de sites moisis */
		if (isset($this->headers['location']) && !isset($this->headers['Location']))
		{
			$this->headers['Location'] = &$this->headers['location'];
		}

		/* Redirection 301, 302 ou 303 */
		if (($this->status === 301 || $this->status === 302 || $this->status === 303) && isset($this->headers['Location']) && $this->redirects-- > 0)
		{
			if (!isset($headers['Cookie']))
			{
				$headers['Cookie'] = '';
			}
			if ($this->headers['Cookies'] !== '' && $this->headers['Cookies'] !== $headers['Cookie'])
			{
				$headers['Cookie'] .= ';'.$this->headers['Cookies'];
			}
			/* On recopie le settings */
			if ($headersOnly === true)
			{
				$headers['headersOnly'] = true;
			}
			/* Une requête POST devient GET, on doit supprimer quelques informations au passage */
			if ($requestType === 'POST')
			{
				unset($headers['Content-Type']);
				unset($headers['Content-Length']);
			}
			if ($this->debugVars[0] === false)
			{
				$this->debugVars[1] .= microtime(true).' DEBUG: Redirection de la requête vers '.$this->headers['Location']."\n";
			}
			/* Le redirect est fait sans le host, on doit l'ajouter */
			if (strpos($this->headers['Location'], 'http') === false)
			{
				/* ltrim() marchera même en cas de «./index.html», qui sera écrit domaine.com/./index.html, et qui marche nativement */
				return $this->get($parsedUrl['scheme'].'://'.$parsedUrl['host'].':'.$parsedUrl['port'].'/'.ltrim($this->headers['Location'], '/'), '', $headers);
			}
			return $this->get($this->headers['Location'], '', $headers);
		}
		/* État de retour OK */
		$status200 = ($this->status >= 200 && $this->status < 300);
		/* Si on fait un HEAD, ou un fake Head, on renvoie illico la réponse trouvée */
		if ($headersOnly === true)
		{
			/* Pour ne pas que la connexion reste ouverte sur le serveur distant, on la détruit */
			fclose($conn);
			unset($this->connections[$parsedUrl['host'].':'.$parsedUrl['port']]);
			return $status200 === true;
		}
		/* En cas de redirection, on renvoie true illico */
		if ($this->status >= 300 && $this->status < 400)
		{
			return true;
		}

		if ($this->memory === true)
		{
			$maxMemory = ini_get('memory_limit');
			if (is_numeric($maxMemory))
			{
				$maxMemory = intval($maxMemory);
			}
			else
			{
				/* Par défaut on aura un «M», donc on met méga-octets sans trop réfléchir */
				$maxMemory = intval($maxMemory) * 1048576;
			}
			/* On garde un minimum de marge */
			$maxMemory *= 0.9;
			if (isset($this->headers['Content-Length']) && intval($this->headers['Content-Length']) > $maxMemory)
			{
				$this->error = 'memory';
				return false;
			}
		}

		/* On remet à 3 le nombre max de redirections */
		$this->redirects = 3;

		$buffer = '';
		/**
		 * Si le contenu est chuncked on le décode,
		 * Le trim est obligatoire car certain serveur envoie un ' chuncked' à la place d'un 'chuncked' (serveur du monde.fr)
		 * http://www.lemonde.fr/economie/article/2011/02/03/les-prix-alimentaires-atteignent-un-pic-historique-en-janvier_1474478_3234.html
		 */
		if (isset($this->headers['Transfer-Encoding']) && trim($this->headers['Transfer-Encoding']) === 'chunked')
		{
			/* Fichier chunked */
			while (feof($conn) === false)
			{
				$length = fgets($conn);
				if ($length === false)
				{
					$this->error = 'chunked 1';
					return false;
				}
				$length = hexdec(trim($length));
				if ($length === 0)
				{
					break;
				}
				$length += 2;
				while ($length > 0)
				{
					$buff = stream_get_line($conn, $length);
					if ($buff === false)
					{
						trigger_error('Erreur sur '.serialize($this), E_USER_WARNING);
						$this->error = 'chunked 2';
						return false;
					}
					$length -= strlen($buff);
					$buffer .= $buff;
				}
				$buffer = substr($buffer, 0, -2);
			}
		}
		else
		{
			/* Fichier avec Content-Length ou sans rien, on traite de la même façon pour ne pas avoir de problème */
			if (!empty($this->headers['Content-Length']))
			{
				$strlen = (int) $this->headers['Content-Length'];
			}
			else
			{
				$strlen = 0;
			}
			while (feof($conn) === false)
			{
				$buff = stream_get_line($conn, $strlen <= 0 ? 4096 : $strlen);
				if ($buff === false)
				{
					/* La dernière lecture plante, alors que la fin de fichier (EOF) n'était pas atteinte, mais EOF juste après cette lecture */
					if (feof($conn) === true)
					{
						break;
					}
					$this->error = 'read error';
					return false;
				}
				$buffer .= $buff;
				/* On fait ce test car certains serveurs n'aiment pas qu'on leur demande trop de caractères à la fois, et en retour on ne peut pas utiliser le Content-Length, alors qu'ils s'y attendent... */
				$strlen -= strlen($buff);
				if ($strlen === 0)
				{
					break;
				}
			}
		}

		/* Le contenu est-il gzippé ? */
		if (isset($this->headers['Content-Encoding']) && ($this->headers['Content-Encoding'] === 'gzip' || $this->headers['Content-Encoding'] === 'deflate'))
		{
			if ($this->headers['Content-Encoding'] === 'gzip')
			{
				$buffer = substr($buffer, 10);
			}
			$this->contents = @gzinflate($buffer);
			if ($this->contents === false)
			{
				$this->error = 'gzinflate';
				return false;
			}
		}
		else
		{
			$this->contents = $buffer;
		}

		return $status200;
	}

	/**
	 * Fonction qui convertit le contenu du fichier en utf8 si besoin et qui decode les entités html (&eacute; ...)
	 *
	 */
	public function forceUtf8()
	{

		if (isset($this->headers['Content-Type']) && preg_match('~charset=([a-zA-Z0-9-]+)~', $this->headers['Content-Type'], $matches) > 0)
		{
			$encoding = $matches[1];
		}
		/**
		 * Si on a un encodage dans le fichier on le remplace
		 * par utf8 pour éviter le double encoding si deux appels
		 * (notamment dans differ)
		 */
		if (preg_match('~(encoding|charset)=(["\']|)([a-zA-Z0-9-]*)\2~', $this->contents, $matches) > 0)
		{
			/**
			 * On donne priorité au header content-type
			 * commme les navigateurs testés : IE/FF/Chrome et Opera
			 */
			if (isset($encoding) === false)
			{
				$encoding = $matches[3];
			}
			if ($matches[1] === 'encoding')
			{
				$this->contents = str_replace($matches[0], 'encoding="utf-8"', $this->contents);
			}
			else
			{
				$this->contents = str_replace($matches[0], 'charset=utf-8', $this->contents);
			}
		}
		/* Fix par Yvan : sinon on a un gros souci en base, car on peut tenter d'insérer de l'iso, et paf */
		if (!isset($encoding))
		{
			$encoding = mb_detect_encoding($this->contents, 'UTF-8,ISO-8859-1,ISO-8859-15');
			if ($encoding === false)
			{
				$encoding = 'ISO-8859-1';
			}
		}
		else
		{
			$encoding = strtoupper($encoding);
			$corrections = array('UTF8' => 'UTF-8', 'ISO8859-1' => 'ISO-8859-1', 'ISO8859-15' => 'ISO-8859-15');
			if (isset($corrections[$encoding]) === true)
			{
				$encoding = $corrections[$encoding];
			}
		}
		if ($encoding !== 'UTF-8')
		{
			if (strpos($encoding, 'ISO-8859') === 0)
			{
				$replace = array(
					"\x00euro;",
					"\x00sbquo;",
					"\x00fnof;",
					"\x00bdquo;",
					"\x00hellip;",
					"\x00dagger;",
					"\x00Dagger;",
					"\x00circ;",
					"\x00permil;",
					"\x00Scaron;",
					"\x00lsaquo;",
					"\x00OElig;",
					"\x00#381;",
					"\x00lsquo;",
					"\x00rsquo;",
					"\x00ldquo;",
					"\x00rdquo;",
					"\x00bull;",
					"\x00ndash;",
					"\x00mdash;",
					"\x00tilde;",
					"\x00trade;",
					"\x00scaron;",
					"\x00rsaquo;",
					"\x00oelig;",
					"\x00#382;",
					"\x00#376;");
				/* On remplace tous les charactères qui proviennent pas de l'iso (Windows-1252) */
				$this->contents = str_replace(array(
					"\x80",
					"\x82",
					"\x83",
					"\x84",
					"\x85",
					"\x86",
					"\x87",
					"\x88",
					"\x89",
					"\x8a",
					"\x8b",
					"\x8c",
					"\x8e",
					"\x91",
					"\x92",
					"\x93",
					"\x94",
					"\xbd",
					"\x96",
					"\x97",
					"\x98",
					"\x99",
					"\x9a",
					"\x9b",
					"\x9c",
					"\x9e",
					"\x9f"), $replace, $this->contents);
			}
			else
			{
				$replace = false;
			}
			$this->contents = mb_convert_encoding($this->contents, 'UTF-8', $encoding);
			if ($replace !== false)
			{
				$this->contents = str_replace($replace, array(
					'€',
					"‚",
					"ƒ",
					"„",
					"…",
					"†",
					"‡",
					"ˆ",
					"‰",
					"Š",
					"‹",
					"Œ",
					"Ž",
					"‘",
					"’",
					"“",
					"”",
					"½",
					"–",
					"—",
					"˜",
					"™",
					"š",
					"›",
					"œ",
					"ž",
					'Ÿ'), $this->contents);
			}
		}
		/* On fait une dernière passe utf-8=>utf-8 pour nettoyer encore un peu plus */
		$this->contents = mb_convert_encoding($this->contents, 'UTF-8', 'UTF-8');
		/**
		 * On remplace les caractères ou les suite de caractères bizarre
		 * \xC2\xAD rencontrée sur http://sidewalkhustle.com/fashion/skingraft-gloves/
		 */
		$this->contents = str_replace(array("\xC2\xAD"), '', $this->contents);
	}

	/**
	 * Fonction qui convertit les liens relatifs du contenu en lien absolu
	 *
	 * @param string $string Chaîne à convertir
	 * @param string $url URL absolue de la page
	 */
	public static function forceAbsoluteLink(&$string, $url)
	{
		$url = parse_url($url);
		$dir = dirname($url['path']);
		/* Protection des autohref et activation des src */
		$string = str_replace('autohref=', 'autoreplacehrmeef=', $string);
		$string = preg_replace('~(<object[^>]+\s)data="~', '\\1datasrc="', $string);
		$string = preg_replace('~name="(src|movie)"\s+value="~', 'name="\\1" objectsrc="', $string);
		/* Réécriture des urls de background CSS */
		$string = preg_replace('~url\s*\(\s*(.+?)\s*\)~i', 'csshref="\\1"', $string);
		/* Protection des liens javascript ou absolus */
		$string = preg_replace('~(src|href)=([\'"]?(?:#|http|javascript:))~i', '\\1locked=\\2', $string);
		/* Protection des urls encodées en JSON */
		$string = preg_replace('~(src|href)=([\'"]?\\\)~i', '\\1locked=\\2', $string);
		/* Protection des src et href dans les paramêtres de get */
		$string = preg_replace('~(&|\?|&amp;)(src|href)=~i', '\\1\\2locked=', $string);
		/* Suppression du host */
		$string = preg_replace('~(src|href)=([\'"]?)http://'.$url['host'].'~i', '\\1=\\2', $string);
		/* Transformation des chemins relatifs en chemins absolus */
		$string = preg_replace('~(src|href)=([\'"]?)\./~i', '\\1=\\2', $string);
		$string = preg_replace('~(src|href)=([\'"]?)(?<!/)([^\'"/]+)~i', '\\1=\\2'.$dir.'\\3', $string);
		/* Ajout du host */
		$string = preg_replace('~(src|href)=([\'"]?)/~i', '\\1=\\2http://'.$url['host'].'/', $string);
		/* Déprotection des liens javascript ou absolus */
		$string = preg_replace('~(src|href)locked=~i', '\\1=', $string);
		/* Réécriture des urls de background CSS */
		$string = preg_replace('~csshref="(.+?)"~i', 'url(\\1)', $string);
		$string = str_replace('autoreplacehrmeef=', 'autohref=', $string);
		$string = str_replace('objectsrc="', 'value="', $string);
		$string = str_replace('datasrc="', 'data="', $string);
	}

	/**
	 * Affiche le débug final
	 */
	public function debug($stop = false)
	{
		if (DEV_SERVER === true)
		{
			echo $this->debugVars[1];
		}
		else
		{
			trigger_error('Debug oublié ! '.$this->debugVars[1], E_USER_WARNING);
		}
		$this->debugVars = array((bool) $stop, '');
	}

	/**
	 * Destructeur, permet de fermer proprement les connexions Keep-Alive
	 */
	public function __destruct()
	{
		/* isset() nécessaire à cause du bug PHP 5.2.6 : http://bugs.php.net/bug.php?id=44913 */
		if (isset($this->connections))
		{
			foreach ($this->connections as $conn)
			{
				/* On vérifie que la connexion est effectivement ouverte */
				if (is_resource($conn['c']))
				{
					fclose($conn['c']);
				}
			}
		}
	}

	public static function cleanPath($url)
	{
		$path = parse_url($url, PHP_URL_PATH);
		$newPath = explode('/', $path);
		foreach ($newPath as $d => $v)
		{
			/* On évite le decode quand il n'y a pas de % */
			if (strpos($v, '%') !== false)
			{
				$v = rawurldecode($v);
			}
			$newPath[$d] = rawurlencode($v);
		}
		$newPath = implode('/', $newPath);
		if ($path !== $newPath)
		{
			$url = str_replace($path, $newPath, $url);
		}
		return $url;
	}

	/**
	 * Retourne une url normalisée pour éviter les doubons
	 *
	 * Supprime les espaces et autres caractères non encodé dans le path de l'url
	 * Supprime les fragments
	 * Tri et nettoye les parametres de la QS
	 * Encode tout ce qui est nécéssaire
	 *
	 * @param string $url
	 * @return string
	 */
	public static function normalizeUrl($url)
	{
		$parse = parse_url($url);
		/* On remplace tous les feed:// ou autre par du http */
		if (isset($parse['scheme']) && $parse['scheme'] !== 'http' && $parse['scheme'] !== 'https')
		{
			$url = substr($url, strlen($parse['scheme']));
			$url = 'http'.$url;
		}
		if (isset($parse['path']))
		{
			$newPath = explode('/', $parse['path']);
			foreach ($newPath as $d => $v)
			{
				/* On évite le decode quand il n'y a pas de % */
				if (strpos($v, '%') !== false)
				{
					$v = urldecode($v);
				}
				$newPath[$d] = rawurlencode($v);
			}
			$newPath = implode('/', $newPath);
			if ($parse['path'] !== $newPath)
			{
				$url = str_replace($parse['path'], $newPath, $url);
			}
		}
		if (isset($parse['query']))
		{
			/* On fait les deux commandes suivante pour remplacer les index vide par leurs valeur numérique */
			$new = explode('&', $parse['query']);
			$params = array();
			$increments = array();
			foreach ($new as $i => $p)
			{
				$pos = strpos($p, '=');
				/* Traitement des url ?= */
				if ($pos === 0)
				{
					continue;
				}
				if (false !== ($pos))
				{
					$n = substr($p, 0, $pos);
					$v = rawurlencode(urldecode(substr($p, $pos + 1)));
				}
				else
				{
					$n = $p;
					$v = '';
				}
				$n = urldecode($n);
				/* Si le nom du parametre vaut un tableau on saute (comportement php sur ?[]=10) */
				$pos = strpos($n, '[');
				if ($pos === 0)
				{
					continue;
				}
				/* Si on a un tableau */
				elseif ($pos !== false)
				{
					/* Le trim en deux fois est important pour ne pas supprimer les [] de fin */
					$p = explode('][', rtrim(trim(substr($n, $pos + 1), '['), ']'));
					$depth = count($p);
					$index = mb_strlen($n) - 1;
					/* On traite les urls du genre : ?data[10]toto */
					if ($depth === 1 && $n[$index] !== ']')
					{
						if ($pos === $index)
						{
							unset($p[$depth - 1]);
							$depth--;
						}
						else
						{
							$n = rawurlencode(substr($n, 0, $pos));
						}
					}
					if ($depth > 0)
					{
						$n = rawurlencode(substr($n, 0, $pos));
						$previousIndex = '';
						foreach ($p as $d => $index)
						{
							/* Si l'index contient une balise fermante on vire la suite (comportement de php sur ?b[0]0=0 */
							if (false !== ($pos = strpos($index, ']')))
							{
								$index = substr($index, 0, $pos);
							}
							/* Si l'index est vide on le remplace par le numérique de ce params */
							if ($index === '')
							{
								if (isset($increments[$n][$previousIndex]) === false)
								{
									$increments[$n][$previousIndex] = 0;
								}
								$index = $increments[$n][$previousIndex];
								$increments[$n][$previousIndex]++;
								$previousIndex .= $index;
							}
							elseif (is_numeric($index))
							{
								$previousIndex .= $index;
							}
							else
							{
								$index = rawurlencode($index);
							}
							$p[$d] = $index;
						}
						$n .= '['.implode('][', $p).']';
					}
				}
				else
				{
					$n = rawurlencode($n);
				}
				$params[$n] = $n.'='.$v;
			}
			unset($increments);
			unset($new);
			/* On trie les params pour ne pas que l'ordre change l'url */
			ksort($params);
			$new = implode('&', $params);
			unset($params);
			if ($new !== $parse['query'])
			{
				$url = str_replace('?'.$parse['query'], '?'.$new, $url);
			}
		}
		/* Si on a un fragment # on le supprime */
		if (isset($parse['fragment']))
		{
			$url = str_replace('#'.$parse['fragment'], '', $url, $url);
		}
		return $url;
	}

}

?>
